@@ -2,9 +2,11 @@ import tensorflow as tf
 from deepac.predict import predict_fasta, predict_npy, filter_fasta
 from deepac.nn_train import RCConfig, RCNet
 from deepac.eval.eval import evaluate_reads
+from deepac.eval.eval_ens import evaluate_ensemble
 from deepac.convert import convert_cudnn
 from deepac import preproc
 from deepac.tests import datagen
+from deepac.tests.rctest import compare_rc
 from deepac.builtin_loading import BuiltinLoader
 from deepac import __file__
 from deepac.explain.tests import ExplainTester
@@ -21,8 +23,9 @@ class Tester:
 
     """
 
-    def __init__(self, n_cpus=8, builtin_configs=None, builtin_weights=None,
-                 explain=False, gwpa=False, do_all=False, do_quick=False, keep=False, scale=1, tpu_resolver=None):
+    def __init__(self, n_cpus=8, builtin_configs=None, builtin_weights=None, explain=False, gwpa=False, do_all=False,
+                 do_quick=False, keep=False, scale=1, tpu_resolver=None, input_modes=None, additivity_check=False,
+                 large=False):
         self.n_cpus = n_cpus
         self.builtin_configs = builtin_configs
         self.builtin_weights = builtin_weights
@@ -34,6 +37,15 @@ class Tester:
         self.keep = keep
         self.scale = scale
         self.tpu_resolver = tpu_resolver
+        self.additivity_check = additivity_check
+        self.do_large = large
+        self.test_config = "nn-test-L.ini" if self.do_large else "nn-test.ini"
+        # all by default, unless using a TPU when it defaults to memory
+        self.input_modes = ["memory"] if tpu_resolver is not None and input_modes is None else input_modes
+        # all are true by default, unless input_modes is specified
+        self.input_modes_dict = {"memory": self.input_modes is None or "memory" in input_modes,
+                                 "sequence": self.input_modes is None or "sequence" in input_modes,
+                                 "tfdata": self.input_modes is None or "tfdata" in input_modes}
 
     def run_datagen(self, npy=True, tfrec=True):
         print("TEST: Generating data...")
@@ -54,7 +66,8 @@ class Tester:
 
         gwpatester = None
         if not self.keep:
-            self.run_datagen(npy=True, tfrec=True if self.tpu_resolver is None else False)
+            self.run_datagen(npy=True,
+                             tfrec=self.input_modes_dict["tfdata"])
         print("TEST: Training...")
         self.test_train(quick)
         print("TEST: Predicting...")
@@ -63,11 +76,15 @@ class Tester:
         self.test_eval()
         print("TEST: Converting...")
         self.test_convert()
+        print("TEST: Continuing training...")
+        self.test_train(quick=True, epoch_start=2, epoch_end=4)
         print("TEST: Filtering...")
         self.test_filter()
 
         if self.do_all or self.gwpa:
-            gwpatester = GWPATester(self.n_cpus)
+            gwpatester = GWPATester(self.n_cpus, self.additivity_check)
+            print("X-TEST: gff2genome...")
+            gwpatester.test_gff2genome()
             print("X-TEST: Fragmenting genomes...")
             gwpatester.test_fragment()
             print("X-TEST: Genome-wide phenotype potential map...")
@@ -80,7 +97,7 @@ class Tester:
             gwpatester.test_fenrichment()
 
         if self.do_all or self.explain:
-            explaintester = ExplainTester(self.n_cpus)
+            explaintester = ExplainTester(self.n_cpus, self.additivity_check)
             print("X-TEST: Maxact (DeepBind)...")
             explaintester.test_maxact()
             # SHAP
@@ -116,6 +133,7 @@ class Tester:
             assert (os.path.isfile(os.path.join("deepac-tests", "sample_train_labels.npy"))), "Preprocessing failed."
         if tfrec:
             config['Options']['Use_TFData'] = "True"
+            config['Options']['Do_shuffle'] = "True"
             p = Process(target=preproc.preproc, args=(config,))
             p.start()
             p.join()
@@ -132,6 +150,7 @@ class Tester:
             assert (os.path.isfile(os.path.join("deepac-tests", "sample_val_labels.npy"))), "Preprocessing failed."
         if tfrec:
             config['Options']['Use_TFData'] = "True"
+            config['Options']['Do_shuffle'] = "True"
             p = Process(target=preproc.preproc, args=(config,))
             p.start()
             p.join()
@@ -139,126 +158,162 @@ class Tester:
                                                 "sample_val_data_0-{}.tfrec".format(128*self.scale-1)))),\
                 "TFData Preprocessing failed."
 
-    def test_train(self, quick=False):
+    def test_train(self, quick=False, epoch_start=0, epoch_end=2):
         """Test training."""
         config = configparser.ConfigParser()
-        config.read(os.path.join(os.path.dirname(__file__), "tests", "configs", "nn-test.ini"))
+        config.read(os.path.join(os.path.dirname(__file__), "tests", "configs", self.test_config))
 
-        print("TEST: Training (custom - .npy in memory)...")
-        config['DataLoad']['Use_TFData'] = "False"
-        config['DataLoad']['LoadTrainingByBatch'] = "False"
-        paprconfig = RCConfig(config)
-        self.__config_train(paprconfig).train()
-        assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                            "nn-deepac-test-e001.h5"))), "Training failed."
-        assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                            "nn-deepac-test-e002.h5"))), "Training failed."
-        assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                            "training-deepac-test.csv"))), "Training failed."
+        if self.input_modes_dict["memory"]:
+            print("TEST: Training (custom - .npy in memory)...")
+            config['DataLoad']['Use_TFData'] = "False"
+            config['DataLoad']['LoadTrainingByBatch'] = "False"
+            paprconfig = RCConfig(config)
+            self.__config_train(paprconfig, epoch_start, epoch_end).train()
+            assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
+                                                "deepac-test-e{epoch:03d}.h5".format(epoch=epoch_start+1)))),\
+                "Training failed."
+            assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
+                                                "deepac-test-e{epoch:03d}.h5".format(epoch=epoch_end)))),\
+                "Training failed."
+            assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
+                                                "training-deepac-test.csv"))), "Training failed."
 
-        if self.tpu_resolver is None:
+        if self.input_modes_dict["sequence"]:
             print("TEST: Training (custom - keras sequence)...")
             config['DataLoad']['Use_TFData'] = "False"
             config['DataLoad']['LoadTrainingByBatch'] = "True"
             paprconfig = RCConfig(config)
-            self.__config_train(paprconfig).train()
+            self.__config_train(paprconfig, epoch_start, epoch_end).train()
             assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                                "nn-deepac-test-e001.h5"))), "Training failed."
+                                                "deepac-test-e{epoch:03d}.h5".format(epoch=epoch_start+1)))),\
+                "Training failed."
             assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                                "nn-deepac-test-e002.h5"))), "Training failed."
+                                                "deepac-test-e{epoch:03d}.h5".format(epoch=epoch_end)))),\
+                "Training failed."
             assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
                                                 "training-deepac-test.csv"))), "Training failed."
 
+        if self.input_modes_dict["tfdata"]:
             print("TEST: Training (custom - tfrecord)...")
             config['DataLoad']['Use_TFData'] = "True"
             paprconfig = RCConfig(config)
-            self.__config_train(paprconfig).train()
+            self.__config_train(paprconfig, epoch_start, epoch_end).train()
             assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                                "nn-deepac-test-e001.h5"))), "Training failed."
+                                                "deepac-test-e{epoch:03d}.h5".format(epoch=epoch_start+1)))),\
+                "Training failed."
             assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                                "nn-deepac-test-e002.h5"))), "Training failed."
+                                                "deepac-test-e{epoch:03d}.h5".format(epoch=epoch_end)))),\
+                "Training failed."
             assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
                                                 "training-deepac-test.csv"))), "Training failed."
 
         if not quick:
-            print("TEST: Training (rapid - .npy in memory)...")
-            paprconfig = self.bloader.get_rapid_training_config()
-            paprconfig.use_tf_data = False
-            paprconfig.use_generators_keras = False
-            self.__config_train(paprconfig).train()
-            runname = paprconfig.runname
-            assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                "nn-{}-e001.h5".format(runname)))), "Training failed."
-            assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                "nn-{}-e002.h5".format(runname)))), "Training failed."
-            assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                "training-{}.csv".format(runname)))), "Training failed."
-            if self.tpu_resolver is None:
+            if self.input_modes_dict["memory"]:
+                print("TEST: Training (rapid - .npy in memory)...")
+                paprconfig = self.bloader.get_rapid_training_config()
+                paprconfig.use_tf_data = False
+                paprconfig.use_generators_keras = False
+                self.__config_train(paprconfig, epoch_start, epoch_end).train()
+                runname = paprconfig.runname
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_start+1)))),\
+                    "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_end)))),\
+                    "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
+                                                    "training-{}.csv".format(runname)))), "Training failed."
+            if self.input_modes_dict["sequence"]:
                 print("TEST: Training (rapid - keras sequence)...")
                 paprconfig = self.bloader.get_rapid_training_config()
                 paprconfig.use_tf_data = False
                 paprconfig.use_generators_keras = True
-                self.__config_train(paprconfig).train()
+                self.__config_train(paprconfig, epoch_start, epoch_end).train()
                 runname = paprconfig.runname
-                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                    "nn-{}-e001.h5".format(runname)))), "Training failed."
-                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                    "nn-{}-e002.h5".format(runname)))), "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_start+1)))),\
+                    "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_end)))),\
+                    "Training failed."
                 assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
                                                     "training-{}.csv".format(runname)))), "Training failed."
 
+            if self.input_modes_dict["tfdata"]:
                 print("TEST: Training (rapid - tfrecord)...")
                 paprconfig = self.bloader.get_rapid_training_config()
                 paprconfig.use_tf_data = True
-                self.__config_train(paprconfig).train()
+                self.__config_train(paprconfig, epoch_start, epoch_end).train()
                 runname = paprconfig.runname
-                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                    "nn-{}-e001.h5".format(runname)))), "Training failed."
-                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                    "nn-{}-e002.h5".format(runname)))), "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_start+1)))),\
+                    "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_end)))),\
+                    "Training failed."
                 assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
                                                     "training-{}.csv".format(runname)))), "Training failed."
 
-            print("TEST: Training (sensitive - .npy in memory)...")
-            paprconfig = self.bloader.get_sensitive_training_config()
-            paprconfig.use_tf_data = False
-            paprconfig.use_generators_keras = False
-            self.__config_train(paprconfig).train()
-            runname = paprconfig.runname
-            assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                "nn-{}-e001.h5".format(runname)))), "Training failed."
-            assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                "nn-{}-e002.h5".format(runname)))), "Training failed."
-            assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                "training-{}.csv".format(runname)))), "Training failed."
+            if self.input_modes_dict["memory"]:
+                print("TEST: Training (sensitive - .npy in memory)...")
+                paprconfig = self.bloader.get_sensitive_training_config()
+                paprconfig.use_tf_data = False
+                paprconfig.use_generators_keras = False
+                self.__config_train(paprconfig, epoch_start, epoch_end).train()
+                runname = paprconfig.runname
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_start+1)))),\
+                    "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_end)))),\
+                    "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
+                                                    "training-{}.csv".format(runname)))), "Training failed."
 
-            if self.tpu_resolver is None:
+            if self.input_modes_dict["sequence"]:
                 print("TEST: Training (sensitive - keras sequence)...")
                 paprconfig = self.bloader.get_sensitive_training_config()
                 paprconfig.use_tf_data = False
                 paprconfig.use_generators_keras = True
-                self.__config_train(paprconfig).train()
+                self.__config_train(paprconfig, epoch_start, epoch_end).train()
                 runname = paprconfig.runname
-                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                    "nn-{}-e001.h5".format(runname)))), "Training failed."
-                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                    "nn-{}-e002.h5".format(runname)))), "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_start+1)))),\
+                    "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_end)))),\
+                    "Training failed."
                 assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
                                                     "training-{}.csv".format(runname)))), "Training failed."
 
+            if self.input_modes_dict["tfdata"]:
                 print("TEST: Training (sensitive - tfrecord)...")
                 paprconfig = self.bloader.get_sensitive_training_config()
                 paprconfig.use_tf_data = True
-                self.__config_train(paprconfig).train()
+                self.__config_train(paprconfig, epoch_start, epoch_end).train()
                 runname = paprconfig.runname
-                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                    "nn-{}-e001.h5".format(runname)))), "Training failed."
-                assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
-                                                    "nn-{}-e002.h5".format(runname)))), "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_start+1)))),\
+                    "Training failed."
+                assert (os.path.isfile(os.path.join("deepac-tests", "{runname}-logs".format(runname=runname),
+                                                    "{runname}-e{epoch:03d}.h5".format(runname=runname,
+                                                                                       epoch=epoch_end)))),\
+                    "Training failed."
                 assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
                                                     "training-{}.csv".format(runname)))), "Training failed."
 
-    def __config_train(self, paprconfig):
+    def __config_train(self, paprconfig, epoch_start=0, epoch_end=2):
         """Set sample data paths and compile."""
         if paprconfig.use_tf_data and tf.executing_eagerly():
             paprconfig.x_train_path = os.path.join("deepac-tests", "sample_train_data")
@@ -270,8 +325,8 @@ class Tester:
             paprconfig.y_train_path = os.path.join("deepac-tests", "sample_train_labels.npy")
             paprconfig.x_val_path = os.path.join("deepac-tests", "sample_val_data.npy")
             paprconfig.y_val_path = os.path.join("deepac-tests", "sample_val_labels.npy")
-        paprconfig.epoch_start = 0
-        paprconfig.epoch_end = 2
+        paprconfig.epoch_start = epoch_start
+        paprconfig.epoch_end = epoch_end
         paprconfig.log_superpath = "deepac-tests"
         paprconfig.log_dir = paprconfig.log_superpath + "/{runname}-logs".format(runname=paprconfig.runname)
         paprconfig.set_tpu_resolver(self.tpu_resolver)
@@ -284,9 +339,10 @@ class Tester:
     def test_pred(self, quick=False):
         """Test predicting."""
         print("TEST: Predicting (custom)...")
-        model = tf.keras.models.load_model(os.path.join("deepac-tests", "deepac-test-logs", "nn-deepac-test-e002.h5"))
-        predict_npy(model, os.path.join("deepac-tests", "sample_val_data.npy"),
-                    os.path.join("deepac-tests", "deepac-test-logs", "deepac-test-e002-predictions-sample_val.npy"))
+        model = tf.keras.models.load_model(os.path.join("deepac-tests", "deepac-test-logs", "deepac-test-e002.h5"))
+        compare_rc(model, os.path.join("deepac-tests", "sample_val_data.npy"),
+                   os.path.join("deepac-tests", "deepac-test-logs", "deepac-test-e002-predictions-sample_val.npy"),
+                   replicates=1)
         assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
                                             "deepac-test-e002-predictions-sample_val.npy"))), "Prediction failed."
 
@@ -294,24 +350,18 @@ class Tester:
             print("TEST: Predicting (rapid)...")
             paprconfig = self.bloader.get_rapid_training_config()
             runname = paprconfig.runname
-            if self.tpu_resolver is not None:
-                paprconfig.use_tf_data = False
-                paprconfig.use_generators_keras = False
             model = self.bloader.load_rapid_model(log_path="deepac-tests", tpu_resolver=self.tpu_resolver)
-            predict_npy(model, os.path.join("deepac-tests", "sample_val_data.npy"),
-                        os.path.join("deepac-tests", "{}-logs".format(runname), "val-pred-rapid.npy"))
+            compare_rc(model, os.path.join("deepac-tests", "sample_val_data.npy"),
+                       os.path.join("deepac-tests", "{}-logs".format(runname), "val-pred-rapid.npy"))
             assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
                                                 "val-pred-rapid.npy"))), "Prediction failed."
 
             print("TEST: Predicting (sensitive)...")
             paprconfig = self.bloader.get_sensitive_training_config()
-            if self.tpu_resolver is not None:
-                paprconfig.use_tf_data = False
-                paprconfig.use_generators_keras = False
             runname = paprconfig.runname
             model = self.bloader.load_sensitive_model(log_path="deepac-tests", tpu_resolver=self.tpu_resolver)
-            predict_npy(model, os.path.join("deepac-tests", "sample_val_data.npy"),
-                        os.path.join("deepac-tests", "{}-logs".format(runname), "val-pred-sensitive.npy"))
+            compare_rc(model, os.path.join("deepac-tests", "sample_val_data.npy"),
+                       os.path.join("deepac-tests", "{}-logs".format(runname), "val-pred-sensitive.npy"))
             assert (os.path.isfile(os.path.join("deepac-tests", "{}-logs".format(runname),
                                                 "val-pred-sensitive.npy"))), "Prediction failed."
 
@@ -327,26 +377,50 @@ class Tester:
         assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
                                             "deepac-test_2_sample_val_aupr.png"))), "Evaluation failed."
 
+        config = configparser.ConfigParser()
+        config.read(os.path.join(os.path.dirname(__file__), "tests", "configs", "eval_ens-test.ini"))
+        evaluate_ensemble(config)
+        assert (os.path.isfile(os.path.join("deepac-tests",
+                                            "ens01-metrics.csv"))), "Evaluation failed."
+        assert (os.path.isfile(os.path.join("deepac-tests",
+                                            "ens01_sample_val_auc.png"))), "Evaluation failed."
+        assert (os.path.isfile(os.path.join("deepac-tests",
+                                            "ens01_sample_val_aupr.png"))), "Evaluation failed."
+
     def test_convert(self):
         """Test converting."""
         config = configparser.ConfigParser()
-        config.read(os.path.join(os.path.dirname(__file__), "tests", "configs", "nn-test.ini"))
+        config.read(os.path.join(os.path.dirname(__file__), "tests", "configs", self.test_config))
         config['Devices']['DistStrategy'] = "OneDeviceStrategy"
-        config['Devices']['BuildDevice'] = "CPU:0"
-        convert_cudnn(config, os.path.join("deepac-tests", "deepac-test-logs", "nn-deepac-test-e002.h5"), False)
+        config['Devices']['Device_build'] = "CPU:0"
+        convert_cudnn(config, os.path.join("deepac-tests", "deepac-test-logs", "deepac-test-e002.h5"), False)
         assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                            "nn-deepac-test-e002_converted.h5"))), "Conversion failed."
+                                            "deepac-test-e002_converted.h5"))), "Conversion failed."
         assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
-                                            "nn-deepac-test-e002_weights.h5"))), "Conversion failed."
+                                            "deepac-test-e002_weights.h5"))), "Conversion failed."
+        config['Architecture']['MC_Dropout'] = 'True'
+        convert_cudnn(config, os.path.join("deepac-tests", "deepac-test-logs", "deepac-test-e002_converted.h5"), False)
+        assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
+                                            "deepac-test-e002_converted_converted.h5"))), "Conversion failed."
+        assert (os.path.isfile(os.path.join("deepac-tests", "deepac-test-logs",
+                                            "deepac-test-e002_converted_weights.h5"))), "Conversion failed."
 
     def test_filter(self):
         """Test filtering."""
-        model = tf.keras.models.load_model(os.path.join("deepac-tests", "deepac-test-logs", "nn-deepac-test-e002.h5"))
-        predict_fasta(model, os.path.join("deepac-tests", "sample-val-pos.fasta"),
+        model = tf.keras.models.load_model(os.path.join("deepac-tests", "deepac-test-logs",
+                                                        "deepac-test-e002_converted_converted.h5"))
+        predict_fasta(model, os.path.join("deepac-tests", "sample-test.fasta"),
                       os.path.join("deepac-tests", "deepac-test-logs",
-                                   "deepac-test-e002-predictions-sample_val-pos.npy"))
-        filter_fasta(os.path.join("deepac-tests", "sample-val-pos.fasta"),
+                                   "deepac-test-e002-predictions-sample_test.npy"),
+                      replicates=5)
+        filter_fasta(os.path.join("deepac-tests", "sample-test.fasta"),
                      os.path.join("deepac-tests", "deepac-test-logs",
-                                  "deepac-test-e002-predictions-sample_val-pos.npy"),
-                     os.path.join("deepac-tests", "sample-val-pos-filtered.fasta"))
-        assert (os.path.isfile(os.path.join("deepac-tests", "sample-val-pos-filtered.fasta"))), "Filtering failed."
+                                  "deepac-test-e002-predictions-sample_test.npy"),
+                     os.path.join("deepac-tests", "sample-test-filtered-pos.fasta"),
+                     print_potentials=True,
+                     output_neg=os.path.join("deepac-tests", "sample-test-filtered-neg.fasta"),
+                     confidence_thresh=0.65,
+                     output_undef=os.path.join("deepac-tests", "sample-test-filtered-undef.fasta"),
+                     pred_uncertainty=os.path.join("deepac-tests", "deepac-test-logs",
+                                                   "deepac-test-e002-predictions-sample_test-std.npy"))
+        assert (os.path.isfile(os.path.join("deepac-tests", "sample-test-filtered-pos.fasta"))), "Filtering failed."
